@@ -1,4 +1,5 @@
 import logging
+import os
 import signal
 import socket
 import sys
@@ -156,7 +157,10 @@ class HttpRequestHandler(BaseHTTPRequestHandler):
     def do_POST(self):
         self.log.info("Received POST request " + self.path)
         try:
-            self.handler.post()
+            if self.path == "/shutdown":
+                os.kill(os.getpid(), signal.SIGTERM)
+            else:
+                self.handler.post()
         except socket.error:
             pass
         except Exception as ex:
@@ -211,12 +215,13 @@ class Service:
         )
         self.log: logging.Logger = setup_log(self.log_file)
 
-    def start(self, httpHandler: BaseHttpHandler):
+    def start(self, create_handler: Callable[[], BaseHttpHandler]):
         eureka_client = register_eureka(
             self.eureka_server, self.name, self.port, self.instance
         )
 
         def unregister_eureka(signal: Any, frame: Any):
+            self.log.info("Cleanup before shutdown.")
             self.cleanup()
             eureka_client.stop()
             sys.exit(0)
@@ -224,9 +229,9 @@ class Service:
         signal.signal(signal.SIGINT, unregister_eureka)
         signal.signal(signal.SIGTERM, unregister_eureka)
 
-        httpHandler.set_log(self.log)
-
         def handler(*args: Any) -> BaseHTTPRequestHandler:
+            httpHandler = create_handler()
+            httpHandler.set_log(self.log)
             return HttpRequestHandler(httpHandler, self.log, *args)
 
         httpServer = ThreadingHTTPServer(("", self.port), handler)
